@@ -24,21 +24,21 @@
  * Class to allow throwing wfDeprecated warnings
  * when people use globals that we do not want them to.
  */
-
 class DeprecatedGlobal extends StubObject {
-	protected $realValue, $version;
+	protected $version;
 
-	function __construct( $name, $realValue, $version = false ) {
-		parent::__construct( $name );
-		$this->realValue = $realValue;
+	/**
+	 * @param string $name Global name
+	 * @param callable|string $callback Factory function or class name to construct
+	 * @param bool|string $version Version global was deprecated in
+	 */
+	function __construct( $name, $callback, $version = false ) {
+		parent::__construct( $name, $callback );
 		$this->version = $version;
 	}
 
-	// @codingStandardsIgnoreStart
-	// PSR2.Methods.MethodDeclaration.Underscore
-	// PSR2.Classes.PropertyDeclaration.ScopeMissing
+	// phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore,PSR2.Classes.PropertyDeclaration.ScopeMissing
 	function _newObject() {
-
 		/* Put the caller offset for wfDeprecated as 6, as
 		 * that gives the function that uses this object, since:
 		 * 1 = this function ( _newObject )
@@ -52,7 +52,6 @@ class DeprecatedGlobal extends StubObject {
 		 * rather unlikely.
 		 */
 		wfDeprecated( '$' . $this->global, $this->version, false, 6 );
-		return $this->realValue;
+		return parent::_newObject();
 	}
-	// @codingStandardsIgnoreEnd
 }
