@@ -28,14 +28,17 @@
  * @ingroup FileAbstraction
  */
 class ForeignAPIFile extends File {
+	/** @var bool */
 	private $mExists;
+	/** @var array */
+	private $mInfo = [];
 
-	protected $repoClass = 'ForeignApiRepo';
+	protected $repoClass = ForeignApiRepo::class;
 
 	/**
-	 * @param $title
-	 * @param $repo ForeignApiRepo
-	 * @param $info
+	 * @param Title|string|bool $title
+	 * @param ForeignApiRepo $repo
+	 * @param array $info
 	 * @param bool $exists
 	 */
 	function __construct( $title, $repo, $info, $exists = false ) {
@@ -48,17 +51,20 @@ class ForeignAPIFile extends File {
 	}
 
 	/**
-	 * @param $title Title
-	 * @param $repo ForeignApiRepo
+	 * @param Title $title
+	 * @param ForeignApiRepo $repo
 	 * @return ForeignAPIFile|null
 	 */
 	static function newFromTitle( Title $title, $repo ) {
-		$data = $repo->fetchImageQuery( array(
+		$data = $repo->fetchImageQuery( [
 			'titles' => 'File:' . $title->getDBkey(),
 			'iiprop' => self::getProps(),
 			'prop' => 'imageinfo',
-			'iimetadataversion' => MediaHandler::getMetadataVersion()
-		) );
+			'iimetadataversion' => MediaHandler::getMetadataVersion(),
+			// extmetadata is language-dependant, accessing the current language here
+			// would be problematic, so we just get them all
+			'iiextmetadatamultilang' => 1,
+		] );
 
 		$info = $repo->getImageInfo( $data );
 
@@ -75,6 +81,7 @@ class ForeignAPIFile extends File {
 			} else {
 				$img = new self( $title, $repo, $info, true );
 			}
+
 			return $img;
 		} else {
 			return null;
@@ -86,7 +93,7 @@ class ForeignAPIFile extends File {
 	 * @return string
 	 */
 	static function getProps() {
-		return 'timestamp|user|comment|url|size|sha1|metadata|mime|mediatype';
+		return 'timestamp|user|comment|url|size|sha1|metadata|mime|mediatype|extmetadata';
 	}
 
 	// Dummy functions...
@@ -130,6 +137,7 @@ class ForeignAPIFile extends File {
 		);
 		if ( $thumbUrl === false ) {
 			global $wgLang;
+
 			return $this->repo->getThumbError(
 				$this->getName(),
 				$width,
@@ -138,13 +146,14 @@ class ForeignAPIFile extends File {
 				$wgLang->getCode()
 			);
 		}
+
 		return $this->handler->getTransform( $this, 'bogus', $thumbUrl, $params );
 	}
 
 	// Info we can get from API...
 
 	/**
-	 * @param $page int
+	 * @param int $page
 	 * @return int|number
 	 */
 	public function getWidth( $page = 1 ) {
@@ -152,7 +161,7 @@ class ForeignAPIFile extends File {
 	}
 
 	/**
-	 * @param $page int
+	 * @param int $page
 	 * @return int
 	 */
 	public function getHeight( $page = 1 ) {
@@ -166,21 +175,35 @@ class ForeignAPIFile extends File {
 		if ( isset( $this->mInfo['metadata'] ) ) {
 			return serialize( self::parseMetadata( $this->mInfo['metadata'] ) );
 		}
+
 		return null;
 	}
 
 	/**
-	 * @param $metadata array
-	 * @return array
+	 * @return array|null Extended metadata (see imageinfo API for format) or
+	 *   null on error
+	 */
+	public function getExtendedMetadata() {
+		if ( isset( $this->mInfo['extmetadata'] ) ) {
+			return $this->mInfo['extmetadata'];
+		}
+
+		return null;
+	}
+
+	/**
+	 * @param mixed $metadata
+	 * @return mixed
 	 */
 	public static function parseMetadata( $metadata ) {
 		if ( !is_array( $metadata ) ) {
 			return $metadata;
 		}
-		$ret = array();
+		$ret = [];
 		foreach ( $metadata as $meta ) {
 			$ret[$meta['name']] = self::parseMetadata( $meta['value'] );
 		}
+
 		return $ret;
 	}
 
@@ -199,14 +222,39 @@ class ForeignAPIFile extends File {
 	}
 
 	/**
-	 * @param string $method
-	 * @return int|null|string
+	 * Get short description URL for a file based on the foreign API response,
+	 * or if unavailable, the short URL is constructed from the foreign page ID.
+	 *
+	 * @return null|string
+	 * @since 1.27
 	 */
-	public function getUser( $method = 'text' ) {
-		return isset( $this->mInfo['user'] ) ? strval( $this->mInfo['user'] ) : null;
+	public function getDescriptionShortUrl() {
+		if ( isset( $this->mInfo['descriptionshorturl'] ) ) {
+			return $this->mInfo['descriptionshorturl'];
+		} elseif ( isset( $this->mInfo['pageid'] ) ) {
+			$url = $this->repo->makeUrl( [ 'curid' => $this->mInfo['pageid'] ] );
+			if ( $url !== false ) {
+				return $url;
+			}
+		}
+		return null;
 	}
 
 	/**
+	 * @param string $type
+	 * @return int|null|string
+	 */
+	public function getUser( $type = 'text' ) {
+		if ( $type == 'text' ) {
+			return isset( $this->mInfo['user'] ) ? strval( $this->mInfo['user'] ) : null;
+		} else {
+			return 0; // What makes sense here, for a remote user?
+		}
+	}
+
+	/**
+	 * @param int $audience
+	 * @param User|null $user
 	 * @return null|string
 	 */
 	public function getDescription( $audience = self::FOR_PUBLIC, User $user = null ) {
@@ -214,16 +262,16 @@ class ForeignAPIFile extends File {
 	}
 
 	/**
-	 * @return null|String
+	 * @return null|string
 	 */
 	function getSha1() {
 		return isset( $this->mInfo['sha1'] )
-			? wfBaseConvert( strval( $this->mInfo['sha1'] ), 16, 36, 31 )
+			? Wikimedia\base_convert( strval( $this->mInfo['sha1'] ), 16, 36, 31 )
 			: null;
 	}
 
 	/**
-	 * @return bool|Mixed|string
+	 * @return bool|string
 	 */
 	function getTimestamp() {
 		return wfTimestamp( TS_MW,
@@ -238,9 +286,10 @@ class ForeignAPIFile extends File {
 	 */
 	function getMimeType() {
 		if ( !isset( $this->mInfo['mime'] ) ) {
-			$magic = MimeMagic::singleton();
+			$magic = MediaWiki\MediaWikiServices::getInstance()->getMimeAnalyzer();
 			$this->mInfo['mime'] = $magic->guessTypesForExtension( $this->getExtension() );
 		}
+
 		return $this->mInfo['mime'];
 	}
 
@@ -251,7 +300,8 @@ class ForeignAPIFile extends File {
 		if ( isset( $this->mInfo['mediatype'] ) ) {
 			return $this->mInfo['mediatype'];
 		}
-		$magic = MimeMagic::singleton();
+		$magic = MediaWiki\MediaWikiServices::getInstance()->getMimeAnalyzer();
+
 		return $magic->getMediaType( null, $this->getMimeType() );
 	}
 
@@ -266,7 +316,7 @@ class ForeignAPIFile extends File {
 
 	/**
 	 * Only useful if we're locally caching thumbs anyway...
-	 * @param $suffix string
+	 * @param string $suffix
 	 * @return null|string
 	 */
 	function getThumbPath( $suffix = '' ) {
@@ -275,6 +325,7 @@ class ForeignAPIFile extends File {
 			if ( $suffix ) {
 				$path = $path . $suffix . '/';
 			}
+
 			return $path;
 		} else {
 			return null;
@@ -282,45 +333,42 @@ class ForeignAPIFile extends File {
 	}
 
 	/**
-	 * @return array
+	 * @return string[]
 	 */
 	function getThumbnails() {
 		$dir = $this->getThumbPath( $this->getName() );
-		$iter = $this->repo->getBackend()->getFileList( array( 'dir' => $dir ) );
+		$iter = $this->repo->getBackend()->getFileList( [ 'dir' => $dir ] );
 
-		$files = array();
-		foreach ( $iter as $file ) {
-			$files[] = $file;
+		$files = [];
+		if ( $iter ) {
+			foreach ( $iter as $file ) {
+				$files[] = $file;
+			}
 		}
 
 		return $files;
 	}
 
-	/**
-	 * @see File::purgeCache()
-	 */
-	function purgeCache( $options = array() ) {
+	function purgeCache( $options = [] ) {
 		$this->purgeThumbnails( $options );
 		$this->purgeDescriptionPage();
 	}
 
 	function purgeDescriptionPage() {
-		global $wgMemc, $wgContLang;
+		global $wgContLang;
 
 		$url = $this->repo->getDescriptionRenderUrl( $this->getName(), $wgContLang->getCode() );
 		$key = $this->repo->getLocalCacheKey( 'RemoteFileDescription', 'url', md5( $url ) );
 
-		$wgMemc->delete( $key );
+		ObjectCache::getMainWANInstance()->delete( $key );
 	}
 
 	/**
-	 * @param $options array
+	 * @param array $options
 	 */
-	function purgeThumbnails( $options = array() ) {
-		global $wgMemc;
-
+	function purgeThumbnails( $options = [] ) {
 		$key = $this->repo->getLocalCacheKey( 'ForeignAPIRepo', 'ThumbUrl', $this->getName() );
-		$wgMemc->delete( $key );
+		ObjectCache::getMainWANInstance()->delete( $key );
 
 		$files = $this->getThumbnails();
 		// Give media handler a chance to filter the purge list
@@ -330,7 +378,7 @@ class ForeignAPIFile extends File {
 		}
 
 		$dir = $this->getThumbPath( $this->getName() );
-		$purgeList = array();
+		$purgeList = [];
 		foreach ( $files as $file ) {
 			$purgeList[] = "{$dir}{$file}";
 		}
@@ -340,4 +388,13 @@ class ForeignAPIFile extends File {
 		# Clear out the thumbnail directory if empty
 		$this->repo->quickCleanDir( $dir );
 	}
+
+	/**
+	 * The thumbnail is created on the foreign server and fetched over internet
+	 * @since 1.25
+	 * @return bool
+	 */
+	public function isTransformedLocally() {
+		return false;
+	}
 }
