@@ -119,10 +119,12 @@ $dateFormats = [
 ];
 
 $fallback8bitEncoding = 'iso-8859-2';
+
 $separatorTransformTable = [
-	',' => "\xc2\xa0", // @bug 2749
+	',' => "\xc2\xa0", // T4749
 	'.' => ','
 ];
+$minimumGroupingDigits = 2;
 
 $linkTrail = '/^([a-zęóąśłżźćńĘÓĄŚŁŻŹĆŃ]+)(.*)$/sDu';
 
@@ -317,4 +319,3 @@ $magicWords = [
 	'pagesincategory_pages'     => [ '0', 'strony', 'pages' ],
 	'pagesincategory_files'     => [ '0', 'pliki', 'files' ],
 ];
-
