@@ -81,12 +81,19 @@ TEXT
 			'Disable link table updates. Is faster but leaves the wiki in an inconsistent state'
 		);
 		$this->addOption( 'image-base-path', 'Import files from a specified path', false, true );
+		$this->addOption( 'skip-to', 'Start from nth page by skipping first n-1 pages', false, true );
+		$this->addOption( 'username-interwiki', 'Use interwiki usernames with this prefix', false, true );
+		$this->addOption( 'no-local-users',
+			'Treat all usernames as interwiki. ' .
+			'The default is to assign edits to local users where they exist.',
+			false, false
+		);
 		$this->addArg( 'file', 'Dump file to import [else use stdin]', false );
 	}
 
 	public function execute() {
 		if ( wfReadOnly() ) {
-			$this->error( "Wiki is in read-only mode; you'll need to disable it for import to work.", true );
+			$this->fatalError( "Wiki is in read-only mode; you'll need to disable it for import to work." );
 		}
 
 		$this->reportingInterval = intval( $this->getOption( 'report', 100 ) );
@@ -110,7 +117,8 @@ TEXT
 		}
 
 		$this->output( "Done!\n" );
-		$this->output( "You might want to run rebuildrecentchanges.php to regenerate RecentChanges\n" );
+		$this->output( "You might want to run rebuildrecentchanges.php to regenerate RecentChanges,\n" );
+		$this->output( "and initSiteStats.php to update page and revision counts\n" );
 	}
 
 	function setNsfilter( array $namespaces ) {
@@ -132,11 +140,12 @@ TEXT
 		if ( strval( $ns ) === $namespace && $wgContLang->getNsText( $ns ) !== false ) {
 			return $ns;
 		}
-		$this->error( "Unknown namespace text / index specified: $namespace", true );
+		$this->fatalError( "Unknown namespace text / index specified: $namespace" );
 	}
 
 	/**
 	 * @param Title|Revision $obj
+	 * @throws MWException
 	 * @return bool
 	 */
 	private function skippedNamespace( $obj ) {
@@ -284,21 +293,38 @@ TEXT
 		$source = new ImportStreamSource( $handle );
 		$importer = new WikiImporter( $source, $this->getConfig() );
 
+		// Updating statistics require a lot of time so disable it
+		$importer->disableStatisticsUpdate();
+
 		if ( $this->hasOption( 'debug' ) ) {
 			$importer->setDebug( true );
 		}
 		if ( $this->hasOption( 'no-updates' ) ) {
 			$importer->setNoUpdates( true );
 		}
+		if ( $this->hasOption( 'username-prefix' ) ) {
+			$importer->setUsernamePrefix(
+				$this->getOption( 'username-prefix' ),
+				!$this->hasOption( 'no-local-users' )
+			);
+		}
 		if ( $this->hasOption( 'rootpage' ) ) {
 			$statusRootPage = $importer->setTargetRootPage( $this->getOption( 'rootpage' ) );
 			if ( !$statusRootPage->isGood() ) {
 				// Die here so that it doesn't print "Done!"
-				$this->error( $statusRootPage->getMessage()->text(), 1 );
+				$this->fatalError( $statusRootPage->getMessage()->text() );
 				return false;
 			}
 		}
+		if ( $this->hasOption( 'skip-to' ) ) {
+			$nthPage = (int)$this->getOption( 'skip-to' );
+			$importer->setPageOffset( $nthPage );
+			$this->pageCount = $nthPage - 1;
+		}
 		$importer->setPageCallback( [ $this, 'reportPage' ] );
+		$importer->setNoticeCallback( function ( $msg, $params ) {
+			echo wfMessage( $msg, $params )->text() . "\n";
+		} );
 		$this->importCallback = $importer->setRevisionCallback(
 			[ $this, 'handleRevision' ] );
 		$this->uploadCallback = $importer->setUploadCallback(
@@ -320,5 +346,5 @@ TEXT
 	}
 }
 
-$maintClass = 'BackupReader';
+$maintClass = BackupReader::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
