@@ -23,6 +23,8 @@
 
 require_once __DIR__ . '/Maintenance.php';
 
+use Wikimedia\Rdbms\ResultWrapper;
+
 /**
  * Maintenance script that sends purge requests for pages edited in a date
  * range to squid/varnish.
@@ -65,7 +67,7 @@ class PurgeChangedPages extends Maintenance {
 			}
 		}
 
-		$dbr = $this->getDB( DB_SLAVE );
+		$dbr = $this->getDB( DB_REPLICA );
 		$minTime = $dbr->timestamp( $this->getOption( 'starttime' ) );
 		$maxTime = $dbr->timestamp( $this->getOption( 'endtime' ) );
 
@@ -77,7 +79,7 @@ class PurgeChangedPages extends Maintenance {
 		$stuckCount = 0; // loop breaker
 		while ( true ) {
 			// Adjust bach size if we are stuck in a second that had many changes
-			$bSize = $this->mBatchSize + ( $stuckCount * $this->mBatchSize );
+			$bSize = ( $stuckCount + 1 ) * $this->getBatchSize();
 
 			$res = $dbr->select(
 				[ 'page', 'revision' ],
@@ -188,5 +190,5 @@ class PurgeChangedPages extends Maintenance {
 	}
 }
 
-$maintClass = "PurgeChangedPages";
+$maintClass = PurgeChangedPages::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
